@@ -2,7 +2,7 @@
 // windows/basic_random_access_handle.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2019 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -16,17 +16,11 @@
 #endif // defined(_MSC_VER) && (_MSC_VER >= 1200)
 
 #include "asio/detail/config.hpp"
+#include "asio/windows/basic_overlapped_handle.hpp"
 
 #if defined(ASIO_HAS_WINDOWS_RANDOM_ACCESS_HANDLE) \
   || defined(GENERATING_DOCUMENTATION)
 
-#include <cstddef>
-#include "asio/detail/handler_type_requirements.hpp"
-#include "asio/detail/throw_error.hpp"
-#include "asio/error.hpp"
-#include "asio/windows/basic_handle.hpp"
-#include "asio/windows/random_access_handle_service.hpp"
-
 #include "asio/detail/push_options.hpp"
 
 namespace asio {
@@ -34,93 +28,136 @@ namespace windows {
 
 /// Provides random-access handle functionality.
 /**
- * The windows::basic_random_access_handle class template provides asynchronous
- * and blocking random-access handle functionality.
+ * The windows::basic_random_access_handle class provides asynchronous and
+ * blocking random-access handle functionality.
  *
  * @par Thread Safety
  * @e Distinct @e objects: Safe.@n
  * @e Shared @e objects: Unsafe.
  */
-template <typename RandomAccessHandleService = random_access_handle_service>
+template <typename Executor = executor>
 class basic_random_access_handle
-  : public basic_handle<RandomAccessHandleService>
+  : public basic_overlapped_handle<Executor>
 {
 public:
-  /// (Deprecated: Use native_handle_type.) The native representation of a
-  /// handle.
-  typedef typename RandomAccessHandleService::native_handle_type native_type;
+  /// The type of the executor associated with the object.
+  typedef Executor executor_type;
 
   /// The native representation of a handle.
-  typedef typename RandomAccessHandleService::native_handle_type
+#if defined(GENERATING_DOCUMENTATION)
+  typedef implementation_defined native_handle_type;
+#else
+  typedef asio::detail::win_iocp_handle_service::native_handle_type
     native_handle_type;
+#endif
 
-  /// Construct a basic_random_access_handle without opening it.
+  /// Construct a random-access handle without opening it.
   /**
-   * This constructor creates a random-access handle without opening it. The
-   * handle needs to be opened before data can be written to or read from it.
+   * This constructor creates a random-access handle without opening it.
    *
-   * @param io_service The io_service object that the random-access handle will
-   * use to dispatch handlers for any asynchronous operations performed on the
-   * handle.
+   * @param ex The I/O executor that the random-access handle will use, by
+   * default, to dispatch handlers for any asynchronous operations performed on
+   * the random-access handle.
    */
-  explicit basic_random_access_handle(asio::io_service& io_service)
-    : basic_handle<RandomAccessHandleService>(io_service)
+  explicit basic_random_access_handle(const executor_type& ex)
+    : basic_overlapped_handle<Executor>(ex)
   {
   }
 
-  /// Construct a basic_random_access_handle on an existing native handle.
+  /// Construct a random-access handle without opening it.
+  /**
+   * This constructor creates a random-access handle without opening it. The
+   * handle needs to be opened or assigned before data can be sent or received
+   * on it.
+   *
+   * @param context An execution context which provides the I/O executor that
+   * the random-access handle will use, by default, to dispatch handlers for any
+   * asynchronous operations performed on the random-access handle.
+   */
+  template <typename ExecutionContext>
+  explicit basic_random_access_handle(ExecutionContext& context,
+      typename enable_if<
+        is_convertible<ExecutionContext&, execution_context&>::value,
+        basic_random_access_handle
+      >::type* = 0)
+    : basic_overlapped_handle<Executor>(context)
+  {
+  }
+
+  /// Construct a random-access handle on an existing native handle.
   /**
    * This constructor creates a random-access handle object to hold an existing
    * native handle.
    *
-   * @param io_service The io_service object that the random-access handle will
-   * use to dispatch handlers for any asynchronous operations performed on the
-   * handle.
+   * @param ex The I/O executor that the random-access handle will use, by
+   * default, to dispatch handlers for any asynchronous operations performed on
+   * the random-access handle.
    *
    * @param handle The new underlying handle implementation.
    *
    * @throws asio::system_error Thrown on failure.
    */
-  basic_random_access_handle(asio::io_service& io_service,
+  basic_random_access_handle(const executor_type& ex,
       const native_handle_type& handle)
-    : basic_handle<RandomAccessHandleService>(io_service, handle)
+    : basic_overlapped_handle<Executor>(ex, handle)
+  {
+  }
+
+  /// Construct a random-access handle on an existing native handle.
+  /**
+   * This constructor creates a random-access handle object to hold an existing
+   * native handle.
+   *
+   * @param context An execution context which provides the I/O executor that
+   * the random-access handle will use, by default, to dispatch handlers for any
+   * asynchronous operations performed on the random-access handle.
+   *
+   * @param handle The new underlying handle implementation.
+   *
+   * @throws asio::system_error Thrown on failure.
+   */
+  template <typename ExecutionContext>
+  basic_random_access_handle(ExecutionContext& context,
+      const native_handle_type& handle,
+      typename enable_if<
+        is_convertible<ExecutionContext&, execution_context&>::value
+      >::type* = 0)
+    : basic_overlapped_handle<Executor>(context, handle)
   {
   }
 
 #if defined(ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
-  /// Move-construct a basic_random_access_handle from another.
+  /// Move-construct a random-access handle from another.
   /**
    * This constructor moves a random-access handle from one object to another.
    *
-   * @param other The other basic_random_access_handle object from which the
+   * @param other The other random-access handle object from which the
    * move will occur.
    *
    * @note Following the move, the moved-from object is in the same state as if
-   * constructed using the @c basic_random_access_handle(io_service&)
+   * constructed using the @c basic_random_access_handle(const executor_type&)
    * constructor.
    */
   basic_random_access_handle(basic_random_access_handle&& other)
-    : basic_handle<RandomAccessHandleService>(
-        ASIO_MOVE_CAST(basic_random_access_handle)(other))
+    : basic_overlapped_handle<Executor>(std::move(other))
   {
   }
 
-  /// Move-assign a basic_random_access_handle from another.
+  /// Move-assign a random-access handle from another.
   /**
    * This assignment operator moves a random-access handle from one object to
    * another.
    *
-   * @param other The other basic_random_access_handle object from which the
+   * @param other The other random-access handle object from which the
    * move will occur.
    *
    * @note Following the move, the moved-from object is in the same state as if
-   * constructed using the @c basic_random_access_handle(io_service&)
+   * constructed using the @c basic_random_access_handle(const executor_type&)
    * constructor.
    */
   basic_random_access_handle& operator=(basic_random_access_handle&& other)
   {
-    basic_handle<RandomAccessHandleService>::operator=(
-        ASIO_MOVE_CAST(basic_random_access_handle)(other));
+    basic_overlapped_handle<Executor>::operator=(std::move(other));
     return *this;
   }
 #endif // defined(ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
@@ -159,8 +196,8 @@ public:
       const ConstBufferSequence& buffers)
   {
     asio::error_code ec;
-    std::size_t s = this->get_service().write_some_at(
-        this->get_implementation(), offset, buffers, ec);
+    std::size_t s = this->impl_.get_service().write_some_at(
+        this->impl_.get_implementation(), offset, buffers, ec);
     asio::detail::throw_error(ec, "write_some_at");
     return s;
   }
@@ -187,8 +224,8 @@ public:
   std::size_t write_some_at(uint64_t offset,
       const ConstBufferSequence& buffers, asio::error_code& ec)
   {
-    return this->get_service().write_some_at(
-        this->get_implementation(), offset, buffers, ec);
+    return this->impl_.get_service().write_some_at(
+        this->impl_.get_implementation(), offset, buffers, ec);
   }
 
   /// Start an asynchronous write at the specified offset.
@@ -211,9 +248,9 @@ public:
    *   std::size_t bytes_transferred           // Number of bytes written.
    * ); @endcode
    * Regardless of whether the asynchronous operation completes immediately or
-   * not, the handler will not be invoked from within this function. Invocation
-   * of the handler will be performed in a manner equivalent to using
-   * asio::io_service::post().
+   * not, the handler will not be invoked from within this function. On
+   * immediate completion, invocation of the handler will be performed in a
+   * manner equivalent to using asio::post().
    *
    * @note The write operation may not transmit all of the data to the peer.
    * Consider using the @ref async_write_at function if you need to ensure that
@@ -239,8 +276,15 @@ public:
     // not meet the documented type requirements for a WriteHandler.
     ASIO_WRITE_HANDLER_CHECK(WriteHandler, handler) type_check;
 
-    return this->get_service().async_write_some_at(this->get_implementation(),
-        offset, buffers, ASIO_MOVE_CAST(WriteHandler)(handler));
+    asio::async_completion<WriteHandler,
+      void (asio::error_code, std::size_t)> init(handler);
+
+    this->impl_.get_service().async_write_some_at(
+        this->impl_.get_implementation(), offset,
+        buffers, init.completion_handler,
+        this->impl_.get_implementation_executor());
+
+    return init.result.get();
   }
 
   /// Read some data from the handle at the specified offset.
@@ -278,8 +322,8 @@ public:
       const MutableBufferSequence& buffers)
   {
     asio::error_code ec;
-    std::size_t s = this->get_service().read_some_at(
-        this->get_implementation(), offset, buffers, ec);
+    std::size_t s = this->impl_.get_service().read_some_at(
+        this->impl_.get_implementation(), offset, buffers, ec);
     asio::detail::throw_error(ec, "read_some_at");
     return s;
   }
@@ -307,8 +351,8 @@ public:
   std::size_t read_some_at(uint64_t offset,
       const MutableBufferSequence& buffers, asio::error_code& ec)
   {
-    return this->get_service().read_some_at(
-        this->get_implementation(), offset, buffers, ec);
+    return this->impl_.get_service().read_some_at(
+        this->impl_.get_implementation(), offset, buffers, ec);
   }
 
   /// Start an asynchronous read at the specified offset.
@@ -331,9 +375,9 @@ public:
    *   std::size_t bytes_transferred           // Number of bytes read.
    * ); @endcode
    * Regardless of whether the asynchronous operation completes immediately or
-   * not, the handler will not be invoked from within this function. Invocation
-   * of the handler will be performed in a manner equivalent to using
-   * asio::io_service::post().
+   * not, the handler will not be invoked from within this function. On
+   * immediate completion, invocation of the handler will be performed in a
+   * manner equivalent to using asio::post().
    *
    * @note The read operation may not read all of the requested number of bytes.
    * Consider using the @ref async_read_at function if you need to ensure that
@@ -360,8 +404,15 @@ public:
     // not meet the documented type requirements for a ReadHandler.
     ASIO_READ_HANDLER_CHECK(ReadHandler, handler) type_check;
 
-    return this->get_service().async_read_some_at(this->get_implementation(),
-        offset, buffers, ASIO_MOVE_CAST(ReadHandler)(handler));
+    asio::async_completion<ReadHandler,
+      void (asio::error_code, std::size_t)> init(handler);
+
+    this->impl_.get_service().async_read_some_at(
+        this->impl_.get_implementation(), offset,
+        buffers, init.completion_handler,
+        this->impl_.get_implementation_executor());
+
+    return init.result.get();
   }
 };
 
