@@ -1,15 +1,15 @@
 //
-// Copyright (C) 2012-2016 Codership Oy <info@codership.com>
+// Copyright (C) 2012-2018 Codership Oy <info@codership.com>
 //
 
 #include "saved_state.hpp"
-#include "gu_dbug.h"
-#include "uuid.hpp"
+#include <gu_dbug.h>
+#include <gu_uuid.hpp>
+#include "gu_thread_keys.hpp"
+#include "gu_inttypes.hpp"
 
 #include <fstream>
 
-#define __STDC_FORMAT_MACROS
-#include <inttypes.h>
 #include <sys/file.h>
 #include <fcntl.h>
 
@@ -21,12 +21,13 @@ namespace galera
 
 SavedState::SavedState  (const std::string& file) :
     fs_           (0),
+    filename_     (file),
     uuid_         (WSREP_UUID_UNDEFINED),
     seqno_        (WSREP_SEQNO_UNDEFINED),
     safe_to_bootstrap_(true),
     unsafe_       (0),
     corrupt_      (false),
-    mtx_          (),
+    mtx_          (gu::get_mutex_key(gu::GU_MUTEX_KEY_SAVED_STATE)),
     written_uuid_ (uuid_),
     current_len_  (0),
     total_marks_  (0),
@@ -48,7 +49,7 @@ SavedState::SavedState  (const std::string& file) :
 
     if (!fs_)
     {
-        gu_throw_error(errno)
+        gu_throw_system_error(errno)
             << "Could not open state file for writing: '" << file
             << "'. Check permissions and/or disk space.";
     }
@@ -182,7 +183,7 @@ SavedState::set (const wsrep_uuid_t& u, wsrep_seqno_t s, bool safe_to_bootstrap)
     safe_to_bootstrap_ = safe_to_bootstrap;
 
     if (0 == unsafe_())
-        write_and_flush (u, s, safe_to_bootstrap);
+        write_file (u, s, safe_to_bootstrap);
     else
         log_debug << "Not writing state: unsafe counter is " << unsafe_();
 }
@@ -205,8 +206,8 @@ SavedState::mark_unsafe()
 
         if (written_uuid_ != WSREP_UUID_UNDEFINED)
         {
-            write_and_flush (WSREP_UUID_UNDEFINED, WSREP_SEQNO_UNDEFINED,
-                             safe_to_bootstrap_);
+            write_file (WSREP_UUID_UNDEFINED, WSREP_SEQNO_UNDEFINED,
+                        safe_to_bootstrap_);
         }
     }
 }
@@ -223,12 +224,12 @@ SavedState::mark_safe()
     {
         gu::Lock lock(mtx_); ++total_locks_;
 
-        if (0 == unsafe_() && (written_uuid_ != uuid_ || seqno_ >= 0))
+        if (0 == unsafe_() && (written_uuid_ != uuid_ || seqno_ >= 0) &&
+            !corrupt_)
         {
-            assert(false == corrupt_);
             /* this will write down proper seqno if set() was called too early
              * (in unsafe state) */
-            write_and_flush (uuid_, seqno_, safe_to_bootstrap_);
+            write_file (uuid_, seqno_, safe_to_bootstrap_);
         }
     }
 }
@@ -236,10 +237,6 @@ SavedState::mark_safe()
 void
 SavedState::mark_corrupt()
 {
-    /* Half LONG_MAX keeps us equally far from overflow and underflow by
-       mark_unsafe()/mark_safe() calls */
-    unsafe_ = (std::numeric_limits<long>::max() >> 1);
-
     gu::Lock lock(mtx_); ++total_locks_;
 
     if (corrupt_) return;
@@ -248,13 +245,28 @@ SavedState::mark_corrupt()
     seqno_ = WSREP_SEQNO_UNDEFINED;
     corrupt_ = true;
 
-    write_and_flush (WSREP_UUID_UNDEFINED, WSREP_SEQNO_UNDEFINED,
-                     safe_to_bootstrap_);
+    write_file (WSREP_UUID_UNDEFINED, WSREP_SEQNO_UNDEFINED,
+                safe_to_bootstrap_);
 }
 
 void
-SavedState::write_and_flush(const wsrep_uuid_t& u, const wsrep_seqno_t s,
-                            bool safe_to_bootstrap)
+SavedState::mark_uncorrupt(const wsrep_uuid_t& u, wsrep_seqno_t s)
+{
+    gu::Lock lock(mtx_); ++total_locks_;
+
+    if (!corrupt_) return;
+
+    uuid_    = u;
+    seqno_   = s;
+    unsafe_  = 0;
+    corrupt_ = false;
+
+    write_file (u, s, safe_to_bootstrap_);
+}
+
+void
+SavedState::write_file(const wsrep_uuid_t& u, const wsrep_seqno_t s,
+                       bool safe_to_bootstrap)
 {
     assert (current_len_ <= MAX_SIZE);
 
@@ -263,22 +275,37 @@ SavedState::write_and_flush(const wsrep_uuid_t& u, const wsrep_seqno_t s,
         if (s >= 0) { log_debug << "Saving state: " << u << ':' << s; }
 
         char buf[MAX_SIZE];
-        const gu_uuid_t* const uu(reinterpret_cast<const gu_uuid_t*>(&u));
         int state_len = snprintf (buf, MAX_SIZE - 1,
                                   "# GALERA saved state"
                                   "\nversion: " VERSION
                                   "\nuuid:    " GU_UUID_FORMAT
                                   "\nseqno:   %" PRId64
                                   "\nsafe_to_bootstrap: %d\n",
-                                  GU_UUID_ARGS(uu), s, safe_to_bootstrap);
+                                  GU_UUID_ARGS(&u), s, safe_to_bootstrap);
 
         int write_size;
         for (write_size = state_len; write_size < current_len_; ++write_size)
             buf[write_size] = ' '; // overwrite whatever is there currently
 
         rewind(fs_);
-        fwrite(buf, write_size, 1, fs_);
-        fflush(fs_);
+
+        if (fwrite(buf, write_size, 1, fs_) == 0) {
+            log_warn << "write file(" << filename_ << ") failed("
+                     << strerror(errno) << ")";
+            return;
+        }
+
+        if (fflush(fs_) != 0) {
+            log_warn << "fflush file(" << filename_ << ") failed("
+                     << strerror(errno) << ")";
+            return;
+        }
+
+        if (fsync(fileno(fs_)) < 0) {
+            log_warn << "fsync file(" << filename_ << ") failed("
+                     << strerror(errno) << ")";
+            return;
+        }
 
         current_len_ = state_len;
         written_uuid_ = u;
