@@ -23,6 +23,7 @@
 #include "gu_event_service.hpp"
 #include "wsrep_config_service.h"
 #include "wsrep_node_isolation.h"
+#include "wsrep_connection_monitor_service.h"
 
 #include <cassert>
 
@@ -539,12 +540,12 @@ galera_terminate_trx(wsrep_t*           const gh,
     return repl->terminate_trx(*trx, meta);
 }
 
-extern "C"
-wsrep_status_t galera_certify(wsrep_t*           const gh,
-                              wsrep_conn_id_t    const conn_id,
-                              wsrep_ws_handle_t* const trx_handle,
-                              uint32_t           const flags,
-                              wsrep_trx_meta_t*  const meta)
+extern "C" wsrep_status_t wsrep_certify_v1(wsrep_t* gh,
+                                           wsrep_conn_id_t conn_id,
+                                           wsrep_ws_handle_t* trx_handle,
+                                           uint32_t flags,
+                                           wsrep_trx_meta_t* meta,
+                                           const wsrep_seq_cb_t* seq_cb)
 {
     assert(gh != 0);
     assert(gh->ctx != 0);
@@ -632,7 +633,7 @@ wsrep_status_t galera_certify(wsrep_t*           const gh,
             }
         }
 
-        retval = repl->replicate(trx, meta);
+        retval = repl->replicate(trx, meta, seq_cb);
 
         if (meta)
         {
@@ -705,6 +706,16 @@ wsrep_status_t galera_certify(wsrep_t*           const gh,
     return retval;
 }
 
+extern "C"
+wsrep_status_t galera_certify(wsrep_t*           const gh,
+                              wsrep_conn_id_t    const conn_id,
+                              wsrep_ws_handle_t* const trx_handle,
+                              uint32_t           const flags,
+                              wsrep_trx_meta_t*  const meta)
+{
+  return wsrep_certify_v1(gh, conn_id, trx_handle, flags, meta, nullptr);
+}
+
 
 extern "C"
 wsrep_status_t galera_commit_order_enter(
@@ -1231,7 +1242,7 @@ wsrep_status_t galera_to_execute_start(wsrep_t*                const gh,
 
         if (trx.nbo_end() == false)
         {
-            retval = repl->replicate(trx, meta);
+            retval = repl->replicate(trx, meta, nullptr);
             assert((retval == WSREP_OK && trx.ts() != 0 &&
                     trx.ts()->global_seqno() > 0) ||
                    (retval != WSREP_OK && (trx.ts() == 0  ||
@@ -1656,6 +1667,16 @@ static wsrep_t galera_str = {
 extern "C"
 int wsrep_loader(wsrep_t *hptr);
 
+extern "C"
+int wsrep_init_connection_monitor_service_v1(wsrep_connection_monitor_service_v1_t *connection_monitor_service)
+{
+    return gu::init_connection_monitor_service_v1(connection_monitor_service);
+}
+
+extern "C" void wsrep_deinit_connection_monitor_service_v1()
+{
+    gu::deinit_connection_monitor_service_v1();
+}
 
 extern "C"
 int wsrep_loader(wsrep_t *hptr)
@@ -1699,6 +1720,68 @@ extern "C" void wsrep_deinit_tls_service_v1()
     gu::deinit_tls_service_v1();
 }
 
+template <typename M>
+static wsrep_status_t
+get_membership(wsrep_t* const           gh,
+               wsrep_allocator_cb const alloc,
+               M**                      memb)
+{
+    assert(gh != 0);
+    assert(gh->ctx != 0);
+
+    REPL_CLASS * repl(reinterpret_cast< REPL_CLASS * >(gh->ctx));
+
+    try
+    {
+        return repl->get_membership(alloc, memb);
+    }
+    catch (std::exception& e)
+    {
+        log_error << e.what();
+        return WSREP_NODE_FAIL;
+    }
+}
+
+static wsrep_status_t
+get_membership_v1(wsrep_t* const            gh,
+                  wsrep_allocator_cb const  alloc,
+                  struct wsrep_membership** memb)
+{
+    return get_membership(gh, alloc, memb);
+}
+
+static wsrep_status_t
+get_membership_v2(wsrep_t* const            gh,
+                  wsrep_allocator_cb const  alloc,
+                  struct wsrep_membership_v2** memb)
+{
+    return get_membership(gh, alloc, memb);
+}
+
+extern "C"
+wsrep_status_t wsrep_init_membership_service_v1(
+    struct wsrep_membership_service_v1 *membership_service)
+{
+    membership_service->get_membership = get_membership_v1;
+    return WSREP_OK;
+}
+
+extern "C" void wsrep_deinit_membership_service_v1()
+{
+}
+
+extern "C"
+wsrep_status_t wsrep_init_membership_service_v2(
+    struct wsrep_membership_service_v2 *membership_service)
+{
+    membership_service->get_membership = get_membership_v2;
+    return WSREP_OK;
+}
+
+extern "C" void wsrep_deinit_membership_service_v2()
+{
+}
+
 extern "C"
 int wsrep_init_allowlist_service_v1(wsrep_allowlist_service_v1_t *allowlist_service)
 {
@@ -1912,18 +1995,6 @@ get_membership(wsrep_t* const            gh,
     }
 }
 
-extern "C"
-wsrep_status_t wsrep_init_membership_service_v1(
-    struct wsrep_membership_service_v1 *membership_service)
-{
-    membership_service->get_membership = get_membership;
-    return WSREP_OK;
-}
-
-extern "C" void wsrep_deinit_membership_service_v1()
-{
-}
-
 /*
  * This function may be called from signal handler, so make sure that
  * only 'safe' system calls and library functions are used. See
