@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2021 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2025 Codership Oy <info@codership.com>
  */
 
 #include "GCache.hpp"
@@ -25,6 +25,7 @@ namespace gcache
         gid            = gu::UUID();
         seqno_max      = SEQNO_NONE;
         seqno_released = SEQNO_NONE;
+        seqno_low_     = SEQNO_NONE;
         seqno_locked   = SEQNO_MAX;
         seqno_locked_count = 0;
 
@@ -46,7 +47,8 @@ namespace gcache
         mem       (params.mem_size(), seqno2ptr, params.debug()),
         rb        (pcb, params.rb_name(), params.rb_size(), seqno2ptr, gid,
                    params.debug(), params.recover()),
-        ps        (params.dir_name(),
+        ps        (*this,
+                   params.dir_name(),
                    params.keep_pages_size(),
                    params.page_size(),
                    params.debug(),
@@ -58,10 +60,12 @@ namespace gcache
         seqno_max     (seqno2ptr.empty() ?
                        SEQNO_NONE : seqno2ptr.index_back()),
         seqno_released(seqno_max),
+        seqno_low_    (SEQNO_NONE),
         seqno_locked  (SEQNO_MAX),
         seqno_locked_count(0)
 #ifndef NDEBUG
         ,buf_tracker()
+        ,in_dtor(false)
 #endif
     {}
 
@@ -71,6 +75,9 @@ namespace gcache
         log_debug << "\n" << "GCache mallocs : " << mallocs
                   << "\n" << "GCache reallocs: " << reallocs
                   << "\n" << "GCache frees   : " << frees;
+#ifndef NDEBUG
+        in_dtor = true;
+#endif
     }
 
     /*! prints object properties */
