@@ -128,7 +128,6 @@ gcomm::Conf::register_params(gu::Config& cnf)
     GCOMM_CONF_ADD (BaseHost);
     GCOMM_CONF_ADD (BasePort);
 
-    GCOMM_CONF_ADD_DEFAULT(ProtonetBackend);
     GCOMM_CONF_ADD_DEFAULT(ProtonetVersion);
 
     GCOMM_CONF_ADD        (TcpNonBlocking);
@@ -141,30 +140,30 @@ gcomm::Conf::register_params(gu::Config& cnf)
     GCOMM_CONF_ADD        (GMCastListenAddr);
     GCOMM_CONF_ADD        (GMCastMCastAddr);
     GCOMM_CONF_ADD        (GMCastMCastPort);
-    GCOMM_CONF_ADD_DEFAULT(GMCastMCastTTL);
+    GCOMM_CONF_ADD        (GMCastMCastTTL);
     GCOMM_CONF_ADD        (GMCastMCastAddr);
-    GCOMM_CONF_ADD_DEFAULT(GMCastTimeWait);
-    GCOMM_CONF_ADD_DEFAULT(GMCastPeerTimeout);
+    GCOMM_CONF_ADD        (GMCastTimeWait);
+    GCOMM_CONF_ADD        (GMCastPeerTimeout);
     GCOMM_CONF_ADD        (GMCastMaxInitialReconnectAttempts);
     GCOMM_CONF_ADD        (GMCastPeerAddr);
     GCOMM_CONF_ADD        (GMCastIsolate);
     GCOMM_CONF_ADD_DEFAULT(GMCastSegment);
 
-    GCOMM_CONF_ADD_DEFAULT(EvsVersion);
+    GCOMM_CONF_ADD        (EvsVersion);
     GCOMM_CONF_ADD_DEFAULT(EvsViewForgetTimeout);
     GCOMM_CONF_ADD_DEFAULT(EvsSuspectTimeout);
     GCOMM_CONF_ADD_DEFAULT(EvsInactiveTimeout);
     GCOMM_CONF_ADD_DEFAULT(EvsInactiveCheckPeriod);
     GCOMM_CONF_ADD        (EvsInstallTimeout);
-    GCOMM_CONF_ADD_DEFAULT(EvsKeepalivePeriod);
+    GCOMM_CONF_ADD        (EvsKeepalivePeriod);
     GCOMM_CONF_ADD_DEFAULT(EvsJoinRetransPeriod);
     GCOMM_CONF_ADD_DEFAULT(EvsStatsReportPeriod);
-    GCOMM_CONF_ADD_DEFAULT(EvsDebugLogMask);
-    GCOMM_CONF_ADD_DEFAULT(EvsInfoLogMask);
+    GCOMM_CONF_ADD        (EvsDebugLogMask);
+    GCOMM_CONF_ADD        (EvsInfoLogMask);
     GCOMM_CONF_ADD_DEFAULT(EvsSendWindow);
     GCOMM_CONF_ADD_DEFAULT(EvsUserSendWindow);
-    GCOMM_CONF_ADD_DEFAULT(EvsUseAggregate);
-    GCOMM_CONF_ADD_DEFAULT(EvsCausalKeepalivePeriod);
+    GCOMM_CONF_ADD        (EvsUseAggregate);
+    GCOMM_CONF_ADD        (EvsCausalKeepalivePeriod);
     GCOMM_CONF_ADD_DEFAULT(EvsMaxInstallTimeouts);
     GCOMM_CONF_ADD_DEFAULT(EvsDelayMargin);
     GCOMM_CONF_ADD_DEFAULT(EvsDelayedKeepPeriod);
@@ -176,7 +175,7 @@ gcomm::Conf::register_params(gu::Config& cnf)
     GCOMM_CONF_ADD_DEFAULT(PcIgnoreQuorum);
     GCOMM_CONF_ADD_DEFAULT(PcChecksum);
     GCOMM_CONF_ADD_DEFAULT(PcAnnounceTimeout);
-    GCOMM_CONF_ADD_DEFAULT(PcLinger);
+    GCOMM_CONF_ADD        (PcLinger);
     GCOMM_CONF_ADD_DEFAULT(PcNpvo);
     GCOMM_CONF_ADD        (PcBootstrap);
     GCOMM_CONF_ADD_DEFAULT(PcWaitPrim);
