@@ -11,10 +11,7 @@
 #ifndef _gcs_h_
 #define _gcs_h_
 
-#include "wsrep_membership_service.h"
-
 #include "gcs_gcache.hpp"
-#include <wsrep_ps.h> // pfs declarations
 
 #include <gu_config.hpp>
 #include <gu_buf.h>
@@ -168,13 +165,6 @@ typedef enum gcs_act_type
 }
 gcs_act_type_t;
 
-/*! returns true if action must be in cache */
-static inline bool
-gcs_act_in_cache(gcs_act_type_t const t)
-{
-    return t <= GCS_ACT_CCHANGE && t != GCS_ACT_COMMIT_CUT;
-}
-
 #define GCS_VOTE_REQUEST 1 /* vote request indicator */
 
 /*! String representations of action types */
@@ -219,8 +209,6 @@ struct gcs_action {
     const void*    buf; /*! unlike input, output goes as a single buffer */
     int32_t        size;
     gcs_act_type_t type;
-
-    bool in_cache() const { return gcs_act_in_cache(type); }
 };
 
 std::ostream& operator <<(std::ostream& os, const gcs_action& act);
@@ -234,13 +222,17 @@ std::ostream& operator <<(std::ostream& os, const gcs_action& act);
  * @param act_in    action buffer vector (total size is passed in action)
  * @param action    action struct
  * @param scheduled whether the call was preceded by gcs_schedule()
+ * @param seq_cb    callback struct for signalling the caller once the
+ *                  replication sequence has been established
  * @return          negative error code, action size in case of success
  * @retval -EINTR:  thread was interrupted while waiting to enter the monitor
  */
 extern long gcs_replv (gcs_conn_t*          conn,
                        const struct gu_buf* act_in,
                        struct gcs_action*   action,
-                       bool                 scheduled);
+                       bool                 scheduled,
+                       const wsrep_seq_cb_t* seq_cb
+);
 
 /*! A wrapper for single buffer communication */
 static inline long gcs_repl (gcs_conn_t*        const conn,
@@ -248,7 +240,7 @@ static inline long gcs_repl (gcs_conn_t*        const conn,
                              bool               const scheduled)
 {
     struct gu_buf const buf = { action->buf, action->size };
-    return gcs_replv (conn, &buf, action, scheduled);
+    return gcs_replv (conn, &buf, action, scheduled, nullptr);
 }
 
 /*! @brief Receives an action from group.
@@ -509,21 +501,7 @@ extern void gcs_get_stats (gcs_conn_t *conn, struct gcs_stats* stats);
 /*! flushes stats counters */
 extern void gcs_flush_stats(gcs_conn_t *conn);
 
-int gcs_get_status(gcs_conn_t* conn, gu::Status& status);
-void gcs_get_membership(const gcs_conn_t* conn,
-                        wsrep_allocator_cb alloc,
-                        struct wsrep_membership** memb);
-
-int gcs_fetch_pfs_info (gcs_conn_t*         conn,
-                        wsrep_node_info_t** nodes,
-                        uint32_t*           size,
-                        int32_t*            my_index,
-                        uint32_t            max_version);
-int gcs_fetch_pfs_stat (gcs_conn_t*         conn,
-                        wsrep_node_stat_t** nodes,
-                        uint32_t*           size,
-                        int32_t*            my_index,
-                        uint32_t            max_version);
+void gcs_get_status(gcs_conn_t* conn, gu::Status& status);
 
 /*! A node with this name will be treated as a stateless arbitrator */
 #define GCS_ARBITRATOR_NAME "garb"
