@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2024 Codership Oy <info@codership.com>
  *
  * $Id$
  *
@@ -24,7 +24,6 @@
 #include <gu_logger.hpp>
 #include <gu_serialize.hpp>
 #include <gu_debug_sync.hpp>
-#include <gu_thread_keys.hpp>
 
 #include <string.h> // for mempcpy
 #include <errno.h>
@@ -170,9 +169,7 @@ gcs_core::gcs_core(gu::Config&  conf,
                 core->fifo = gcs_fifo_lite_create (CORE_FIFO_LEN,
                                                    sizeof (core_act_t));
                 if (core->fifo) {
-                    gu_mutex_init(
-                        gu::get_mutex_key(gu::GU_MUTEX_KEY_GCS_CORE_SEND),
-                        &core->send_lock);
+                    gu_mutex_init  (&core->send_lock, NULL);
                     core->proto_ver = -1;
                     // ^^^ shall be bumped in gcs_group_act_conf()
                     core->state = CORE_CLOSED;
@@ -368,8 +365,6 @@ gcs_core_send (gcs_core_t*          const conn,
 
     assert (action != NULL);
     assert (act_size > 0);
-    /* actions passing this way will be put in cache */
-    assert (gcs_act_in_cache(act_type));
 
     /*
      * Action header will be replicated with every message.
@@ -681,7 +676,7 @@ core_handle_act_msg (gcs_core_t*          core,
                 ret = gcs_group_handle_state_request (group, act);
                 assert (ret <= 0 || ret == act->act.buf_len);
 #ifdef GCS_FOR_GARB
-                if (ret < 0) gu_fatal ("Handling state request failed: %ld",ret);
+                if (ret < 0) gu_fatal ("Handling state request failed: %zd",ret);
                 act->act.buf = NULL;
             }
             else {
@@ -921,7 +916,7 @@ core_handle_comp_msg (gcs_core_t*          const core,
                     assert(act->buf == NULL);
                     assert(act->buf_len == 0);
                     act->type = GCS_ACT_ERROR;
-                    gu_debug("comp msg error in core %ld", -ret);
+                    gu_debug("comp msg error in core %zd", -ret);
                 }
             }
             else {                               // regular non-prim
@@ -931,7 +926,7 @@ core_handle_comp_msg (gcs_core_t*          const core,
             if (GCS_GROUP_NON_PRIMARY == ret) { // no error in comp msg
                 ret = gcs_group_act_conf (group, rcvd, &core->proto_ver);
                 if (ret < 0) {
-                    gu_fatal ("Failed create NON-PRIM CONF action: %ld (%s)",
+                    gu_fatal ("Failed create NON-PRIM CONF action: %zd (%s)",
                               ret, strerror (-ret));
                     assert (0);
                     ret = -ENOTRECOVERABLE;
@@ -953,7 +948,7 @@ core_handle_comp_msg (gcs_core_t*          const core,
         assert(0);
         // fall through
     default:
-        gu_fatal ("Failed to handle component message: %ld (%s)!",
+        gu_fatal ("Failed to handle component message: %zd (%s)!",
                   ret, strerror (-ret));
         assert(0);
     }
@@ -1603,8 +1598,8 @@ gcs_core_caused (gcs_core_t* core, gu::GTID& gtid)
     gu_cond_t    cond;
     causal_act_t act = {&act_id, &act_uuid, &error, &mtx, &cond};
 
-    gu_mutex_init (gu::get_mutex_key(gu::GU_MUTEX_KEY_GCS_CORE_CAUSED), &mtx);
-    gu_cond_init  (gu::get_cond_key(gu::GU_COND_KEY_GCS_CORE_CAUSED), &cond);
+    gu_mutex_init (&mtx, NULL);
+    gu_cond_init  (&cond, NULL);
     gu_mutex_lock (&mtx);
     {
         long ret = core_msg_send_retry (core,
@@ -1657,67 +1652,16 @@ gcs_core_param_get (gcs_core_t* core, const char* key)
     }
 }
 
-int gcs_core_get_status(gcs_core_t* core, gu::Status& status)
+void gcs_core_get_status(gcs_core_t* core, gu::Status& status)
 {
-    int rc = 0;
     if (gu_mutex_lock(&core->send_lock))
-    {
-        return -ENOTRECOVERABLE;
-    }
+        gu_throw_fatal << "could not lock mutex";
     if (core->state < CORE_CLOSED)
     {
         gcs_group_get_status(&core->group, status);
         core->backend.status_get(&core->backend, status);
     }
     gu_mutex_unlock(&core->send_lock);
-    return rc;
-}
-
-int gcs_core_fetch_pfs_info(gcs_core_t*         core,
-                            wsrep_node_info_t** nodes,
-                            uint32_t*           size,
-                            int32_t*            my_index,
-                            uint32_t            max_version)
-{
-    int rc = -ENOTCONN;
-    if (gu_mutex_lock(&core->send_lock))
-    {
-        return -ENOTRECOVERABLE;
-    }
-    if (core->state < CORE_CLOSED)
-    {
-        rc = gcs_group_fetch_pfs_info(&core->group, nodes, size, my_index,
-                                      max_version);
-    }
-    gu_mutex_unlock(&core->send_lock);
-    return rc;
-}
-
-int gcs_core_fetch_pfs_stat(gcs_core_t*         core,
-                            wsrep_node_stat_t** nodes,
-                            uint32_t*           size,
-                            int32_t*            my_index,
-                            uint32_t            max_version)
-{
-    int rc = -ENOTCONN;
-    if (gu_mutex_lock(&core->send_lock))
-    {
-        return -ENOTRECOVERABLE;
-    }
-    if (core->state < CORE_CLOSED)
-    {
-        rc = gcs_group_fetch_pfs_stat(&core->group, nodes, size, my_index,
-                                      max_version);
-    }
-    gu_mutex_unlock(&core->send_lock);
-    return rc;
-}
-
-void gcs_core_get_membership(const gcs_core_t* const   core,
-                             wsrep_allocator_cb const  alloc,
-                             struct wsrep_membership** memb)
-{
-    gcs_group_get_membership(core->group, alloc, memb);
 }
 
 void gcs_core_get_protocols(gcs_core_t* core, int& appl, int& repl, int& gcs)
@@ -1763,10 +1707,4 @@ gcs_core_get_fifo (gcs_core_t* core)
     return core->fifo;
 }
 
-gcache_t*
-gcs_core_get_gcache(gcs_core_t* core)
-{
-    return core->cache;
-}
-
 #endif /* GCS_CORE_TESTING */
