@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2025 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -8,6 +8,7 @@
 #include "gcs_gcache.hpp"
 #include "gcs_priv.hpp"
 #include "gcs_code_msg.hpp"
+#include "gcs_error.hpp"
 
 #include <gu_logger.hpp>
 #include <gu_macros.hpp>
@@ -19,16 +20,22 @@
 
 #include <errno.h>
 
+#include <cinttypes>
 #include <limits>
 
+std::string const GCS_STATELESS_KEY("gcs.stateless");
+bool        const GCS_STATELESS_DEFAULT(false);
 std::string const GCS_VOTE_POLICY_KEY("gcs.vote_policy");
 uint8_t     const GCS_VOTE_POLICY_DEFAULT(0);
 
-void gcs_group_register(gu::Config* cnf)
+void gcs_group::register_params(gu::Config& cnf)
 {
-    cnf->add(GCS_VOTE_POLICY_KEY,
-             gu::Config::Flag::read_only |
-             gu::Config::Flag::type_integer);
+    cnf.add(GCS_STATELESS_KEY,
+            gu::Config::Flag::read_only |
+            gu::Config::Flag::type_bool);
+    cnf.add(GCS_VOTE_POLICY_KEY,
+            gu::Config::Flag::read_only |
+            gu::Config::Flag::type_integer);
 }
 
 const char* gcs_group_state_str[GCS_GROUP_STATE_MAX] =
@@ -39,6 +46,11 @@ const char* gcs_group_state_str[GCS_GROUP_STATE_MAX] =
     "PRIMARY"
 };
 
+static bool
+group_conf_stateless_flag(gu::Config& cnf)
+{
+    return cnf.get(GCS_STATELESS_KEY, GCS_STATELESS_DEFAULT);
+}
 
 uint8_t gcs_group_conf_to_vote_policy(gu::Config& cnf)
 {
@@ -54,49 +66,51 @@ uint8_t gcs_group_conf_to_vote_policy(gu::Config& cnf)
     return i;
 }
 
-int
-gcs_group_init (gcs_group_t* group, gu::Config* const cnf, gcache_t* const cache,
-                const char* node_name, const char* inc_addr,
-                gcs_proto_t const gcs_proto_ver, int const repl_proto_ver,
-                int const appl_proto_ver)
-{
-    // here we also create default node instance.
-    group->cache        = cache;
-    group->act_id_      = GCS_SEQNO_ILL;
-    group->conf_id      = GCS_SEQNO_ILL;
-    group->state_uuid   = GU_UUID_NIL;
-    group->group_uuid   = GU_UUID_NIL;
-    group->num          = 0;
-    group->my_idx       = -1;
-    group->my_name      = strdup(node_name ? node_name : NODE_NO_NAME);
-    group->my_address   = strdup(inc_addr  ? inc_addr  : NODE_NO_ADDR);
-    group->state        = GCS_GROUP_NON_PRIMARY;
-    group->last_applied = group->act_id_;
-    group->last_node    = -1;
-    group->vote_request_seqno = GCS_NO_VOTE_SEQNO;
-    group->vote_result  = (VoteResult){ GCS_NO_VOTE_SEQNO, 0 };
-    group->vote_history = new VoteHistory;
-    group->vote_policy  = gcs_group_conf_to_vote_policy(*cnf);
-    group->frag_reset   = true; // just in case
-    group->nodes        = NULL;
-    group->prim_uuid    = GU_UUID_NIL;
-    group->prim_seqno   = GCS_SEQNO_ILL;
-    group->prim_num     = 0;
-    group->prim_state   = GCS_NODE_STATE_NON_PRIM;
-    group->prim_gcs_ver  = 0;
-    group->prim_repl_ver = 0;
-    group->prim_appl_ver = 0;
+gcs_group::gcs_group(gu::Config&  cnf,
+                     gcache_t*    cache,
+                     const char*  node_name, ///< can be null
+                     const char*  inc_addr,  ///< can be null
+                     gcs_proto_t  gcs_proto_ver,
+                     int          repl_proto_ver,
+                     int          appl_proto_ver)
+    :
+    memb_mtx_     (gu::get_mutex_key(gu::GU_MUTEX_KEY_GCS_MEMBERSHIP)),
+    memb_epoch_   (GCS_SEQNO_ILL),
+    cache         (cache),
+    cnf           (cnf),
+    act_id_       (GCS_SEQNO_ILL),
+    conf_id       (GCS_SEQNO_ILL),
+    state_uuid    (GU_UUID_NIL),
+    group_uuid    (GU_UUID_NIL),
+    num           (0),
+    my_idx        (-1),
+    my_name       (strdup(node_name ? node_name : NODE_NO_NAME)),
+    my_address    (strdup(inc_addr  ? inc_addr  : NODE_NO_ADDR)),
+    state         (GCS_GROUP_NON_PRIMARY),
+    last_applied  (act_id_),
+    last_node     (-1),
+    vote_request_seqno (GCS_NO_VOTE_SEQNO),
+    vote_result   ((VoteResult){ GCS_NO_VOTE_SEQNO, 0 }),
+    vote_history  (),
+    vote_policy   (gcs_group_conf_to_vote_policy(cnf)),
+    frag_reset    (true), // just in case
+    stateless     (group_conf_stateless_flag(cnf)),
+    nodes         (NULL),
+    prim_uuid     (GU_UUID_NIL),
+    prim_seqno    (GCS_SEQNO_ILL),
+    prim_num      (0),
+    prim_state    (GCS_NODE_STATE_NON_PRIM),
+    prim_gcs_ver  (0),
+    prim_repl_ver (0),
+    prim_appl_ver (0),
 
-    *(gcs_proto_t*)&group->gcs_proto_ver = gcs_proto_ver;
-    *(int*)&group->repl_proto_ver = repl_proto_ver;
-    *(int*)&group->appl_proto_ver = appl_proto_ver;
+    gcs_proto_ver (gcs_proto_ver),
+    repl_proto_ver(repl_proto_ver),
+    appl_proto_ver(appl_proto_ver),
 
-    group->quorum = GCS_QUORUM_NON_PRIMARY;
-
-    group->last_applied_proto_ver = -1;
-
-    return 0;
-}
+    quorum        (GCS_QUORUM_NON_PRIMARY),
+    last_applied_proto_ver(-1)
+{}
 
 int
 gcs_group_init_history (gcs_group_t*    group,
@@ -138,19 +152,20 @@ group_nodes_init (const gcs_group_t* group, const gcs_comp_msg_t* comp)
 
             if (my_idx != i) {
                 gcs_node_init (&ret[i], group->cache, memb->id,
-                               NULL, NULL, -1, -1, -1, memb->segment);
+                               NULL, NULL, -1, -1, -1, memb->segment, false);
             }
             else { // this node
                 gcs_node_init (&ret[i], group->cache, memb->id,
                                group->my_name, group->my_address,
                                group->gcs_proto_ver, group->repl_proto_ver,
-                               group->appl_proto_ver, memb->segment);
+                               group->appl_proto_ver, memb->segment,
+                               group->stateless);
             }
             assert(ret[i].last_applied == GCS_SEQNO_NIL);
         }
     }
     else {
-        gu_error ("Could not allocate %ld x %z bytes", nodes_num,
+        gu_error ("Could not allocate %ld x %zu bytes", nodes_num,
                   sizeof(gcs_node_t));
     }
     return ret;
@@ -183,12 +198,16 @@ gcs_group_free (gcs_group_t* group)
 {
     if (group->my_name)    free ((char*)group->my_name);
     if (group->my_address) free ((char*)group->my_address);
-    delete group->vote_history;
 
     gu::Lock lock(group->memb_mtx_);
     group_nodes_free (group);
 }
 
+gcs_group::~gcs_group()
+{
+    gcs_group_free(this);
+}
+
 /* Reset nodes array without breaking the statistics */
 static inline void
 group_nodes_reset (gcs_group_t* group)
@@ -207,24 +226,27 @@ group_nodes_reset (gcs_group_t* group)
 }
 
 /*! @return false
- *  if the node is arbitrator and must not be counted in commit cut */
+ *  if the node is stateless and must not be counted in commit cut */
 static inline bool
-group_count_arbitrator(const gcs_group_t& group, const gcs_node_t& node)
+group_count_stateless(const gcs_group_t& group, const gcs_node_t& node)
 {
-    return (!(group.quorum.gcs_proto_ver > 0 && node.arbitrator));
+    return (!(group.quorum.gcs_proto_ver > 0 && node.stateless));
 }
 
 /*! @return true if the node should be counted in commit cut calculations */
 static inline bool
 group_count_last_applied(const gcs_group_t& group, const gcs_node_t& node)
 {
-    return (node.count_last_applied && group_count_arbitrator(group, node));
+    return (node.count_last_applied && group_count_stateless(group, node));
 }
 
 /* Find node with the smallest last_applied */
 static inline void
 group_redo_last_applied (gcs_group_t* group)
 {
+    /* protocols 2-3-4 had error in commit cut recalculation */
+    bool const proto_cond(group->quorum.gcs_proto_ver > 4 ||
+                          group->quorum.gcs_proto_ver < 2);
     gu_seqno_t last_applied = GU_LLONG_MAX;
     int        last_node    = -1;
     int        n;
@@ -260,7 +282,7 @@ group_redo_last_applied (gcs_group_t* group)
                          << " below the current " << group->last_applied;
             }
 #endif /* NDEBUG */
-            if (seqno >= group->last_applied || group->quorum.gcs_proto_ver < 2)
+            if (proto_cond || seqno >= group->last_applied)
             {
                 last_applied = seqno;
                 last_node    = n;
@@ -286,10 +308,17 @@ group_redo_last_applied (gcs_group_t* group)
     }
 
     if (gu_likely (last_node >= 0)) {
-        assert(last_applied >= group->last_applied ||
-               group->quorum.gcs_proto_ver < 2);
-        group->last_applied = last_applied;
-        group->last_node    = last_node;
+        assert(last_applied < GU_LLONG_MAX);
+        assert(last_applied >= group->last_applied || proto_cond);
+        /* make sure group-wide last applied is monotonically increasing:
+         * newly SYNCED node may temporarily result in lower last_applied. */
+        if (last_applied > group->last_applied ||
+            group->quorum.gcs_proto_ver < 2)
+        {
+            group->last_applied = last_applied;
+        }
+        /* should always be the most lagging node to trigger recalculation ASAP*/
+        group->last_node = last_node;
     }
 
     log_debug << "final last_applied on " << group->nodes[group->my_idx].name
@@ -321,7 +350,7 @@ group_go_non_primary (gcs_group_t* group)
     // what else? Do we want to change anything about the node here?
 }
 
-static void
+static int
 group_check_proto_ver(gcs_group_t* group)
 {
     assert(group->quorum.primary); // must be called only on primary CC
@@ -332,8 +361,8 @@ group_check_proto_ver(gcs_group_t* group)
 #define GROUP_CHECK_NODE_PROTO_VER(LEVEL)                               \
     if (node.LEVEL < group->quorum.LEVEL) {                             \
         gu_fatal("Group requested %s: %d, max supported by this node: %d." \
-                 "Upgrade the node before joining this group."          \
-                 "Need to abort.",                                      \
+                 " Upgrade the node before joining this group."         \
+                 " Must abort.",                                        \
                  #LEVEL, group->quorum.LEVEL, node.LEVEL);              \
         fail = true;                                                    \
     }
@@ -344,12 +373,14 @@ group_check_proto_ver(gcs_group_t* group)
 
 #undef GROUP_CHECK_NODE_PROTO_VER
 
-    if (fail) gu_abort();
+    if (fail) return -ENOTRECOVERABLE;
+
+    return 0;
 }
 
 static const char group_empty_id[GCS_COMP_MEMB_ID_MAX_LEN + 1] = { 0, };
 
-static void
+static int
 group_check_donor (gcs_group_t* group)
 {
     gcs_node_state_t const my_state = group->nodes[group->my_idx].status;
@@ -365,20 +396,20 @@ group_check_donor (gcs_group_t* group)
             if (i != group->my_idx &&
                 !memcmp (donor_id, group->nodes[i].id,
                          sizeof (group->nodes[i].id)))
-                return;
+                return 0;
         }
 
         gu_warn ("Donor %s is no longer in the group. State transfer cannot "
-                 "be completed, need to abort. Aborting...", donor_id);
+                 "be completed, need to abort.", donor_id);
 
-        gu_abort();
+        return -ENOTRECOVERABLE;
     }
 
-    return;
+    return 0;
 }
 
 /*! Processes state messages and sets group parameters accordingly */
-static void
+static int
 group_post_state_exchange (gcs_group_t* group)
 {
     assert(group->memb_mtx_.locked());
@@ -399,7 +430,7 @@ group_post_state_exchange (gcs_group_t* group)
             (new_exchange &&
              gu_uuid_compare (&group->state_uuid,
                               gcs_state_msg_uuid(states[i]))))
-            return; // not all states from THIS state exch. received, wait
+            return 0; // not all states from THIS state exch. received, wait
     }
     gu_debug ("STATE EXCHANGE: " GU_UUID_FORMAT " complete.",
               GU_UUID_ARGS(&group->state_uuid));
@@ -417,7 +448,7 @@ group_post_state_exchange (gcs_group_t* group)
     }
     else {
         gu_fatal ("Negative quorum version: %d", quorum->version);
-        gu_abort();
+        return -ENOTRECOVERABLE;
     }
 
     // Update each node state based on quorum outcome:
@@ -439,7 +470,7 @@ group_post_state_exchange (gcs_group_t* group)
                          (long long)group->act_id_, (long long)quorum->act_id,
                          (long long)(group->act_id_ - quorum->act_id));
                 group->state  = GCS_GROUP_INCONSISTENT;
-                return;
+                return 0;
             }
             group->state      = GCS_GROUP_PRIMARY;
             group->act_id_    = quorum->act_id;
@@ -492,10 +523,10 @@ group_post_state_exchange (gcs_group_t* group)
     gu_info ("Quorum results:"
              "\n\tversion    = %u,"
              "\n\tcomponent  = %s,"
-             "\n\tconf_id    = %lld,"
-             "\n\tmembers    = %d/%d (joined/total),"
-             "\n\tact_id     = %lld,"
-             "\n\tlast_appl. = %lld,"
+             "\n\tconf_id    = %" PRId64 ","
+             "\n\tmembers    = %ld/%ld (joined/total),"
+             "\n\tact_id     = %" PRId64 ","
+             "\n\tlast_appl. = %" PRId64 ","
              "\n\tprotocols  = %d/%d/%d (gcs/repl/appl),"
              "\n\tvote policy= %d,"
              "\n\tgroup UUID = " GU_UUID_FORMAT,
@@ -510,19 +541,22 @@ group_post_state_exchange (gcs_group_t* group)
              int(quorum->vote_policy),
              GU_UUID_ARGS(&quorum->group_uuid));
 
-    if (quorum->primary) group_check_proto_ver(group);
-    group_check_donor(group);
+    if (quorum->primary) {
+        int const err(group_check_proto_ver(group));
+        if (err) return err;
+    }
+    return group_check_donor(group);
 }
 
 // does basic sanity check of the component message (in response to #145)
-static void
+static int
 group_check_comp_msg (bool prim, long my_idx, long members)
 {
     if (my_idx >= 0) {
-        if (my_idx < members) return;
+        if (my_idx < members) return 0;
     }
     else {
-        if (!prim && (0 == members)) return;
+        if (!prim && (0 == members)) return 0;
     }
 
     gu_fatal ("Malformed component message from backend: "
@@ -530,7 +564,7 @@ group_check_comp_msg (bool prim, long my_idx, long members)
               prim ? "PRIMARY" : "NON-PRIMARY", my_idx, members);
 
     assert (0);
-    gu_abort ();
+    return -ENOTRECOVERABLE;
 }
 
 gcs_group_state_t
@@ -545,7 +579,10 @@ gcs_group_handle_comp_msg (gcs_group_t* group, const gcs_comp_msg_t* comp)
     const long new_my_idx    = gcs_comp_msg_self     (comp);
     const long new_nodes_num = gcs_comp_msg_num      (comp);
 
-    group_check_comp_msg (prim_comp, new_my_idx, new_nodes_num);
+    {
+        int const err(group_check_comp_msg(prim_comp, new_my_idx,new_nodes_num));
+        if (err) return gcs_group_state_t(err);
+    }
 
     if (new_my_idx >= 0) {
         gu_info ("New COMPONENT: primary = %s, bootstrap = %s, my_idx = %ld, "
@@ -555,7 +592,7 @@ gcs_group_handle_comp_msg (gcs_group_t* group, const gcs_comp_msg_t* comp)
         new_nodes = group_nodes_init (group, comp);
 
         if (!new_nodes) {
-            gu_fatal ("Could not allocate memory for %ld-node component.",
+            gu_fatal ("Could not allocate memory for %d-node component.",
                       gcs_comp_msg_num (comp));
             assert(0);
             return (gcs_group_state_t)-ENOMEM;
@@ -685,7 +722,8 @@ gcs_group_handle_comp_msg (gcs_group_t* group, const gcs_comp_msg_t* comp)
             if (GCS_GROUP_PRIMARY == group->state) {
                 /* since we don't have any new nodes since last PRIMARY,
                    we skip state exchange */
-                group_post_state_exchange (group);
+                int const err(group_post_state_exchange(group));
+                if (err) return gcs_group_state_t(err);
             }
         }
 
@@ -711,7 +749,7 @@ gcs_group_handle_uuid_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
     }
     else {
         gu_warn ("Stray state UUID msg: " GU_UUID_FORMAT
-                 " from node %ld (%s), current group state %s",
+                 " from node %d (%s), current group state %s",
                  GU_UUID_ARGS((gu_uuid_t*)msg->buf),
                  msg->sender_idx, group->nodes[msg->sender_idx].name,
                  gcs_group_state_str[group->state]);
@@ -744,12 +782,13 @@ gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
                     gu::Lock lock(group->memb_mtx_);
                     group->memb_epoch_ = group->act_id_;
                     gcs_node_record_state(&group->nodes[msg->sender_idx], state);
-                    group_post_state_exchange (group);
+                    int const err(group_post_state_exchange(group));
+                    if (err) return gcs_group_state_t(err);
                 }
             }
             else {
                 gu_debug ("STATE EXCHANGE: stray state msg: " GU_UUID_FORMAT
-                          " from node %ld (%s), current state UUID: "
+                          " from node %d (%s), current state UUID: "
                           GU_UUID_FORMAT,
                           GU_UUID_ARGS(state_uuid),
                           msg->sender_idx, gcs_state_msg_name(state),
@@ -760,7 +799,7 @@ gcs_group_handle_state_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
             }
         }
         else {
-            gu_warn ("Could not parse state message from node %d",
+            gu_warn ("Could not parse state message from node %d, %s",
                      msg->sender_idx, group->nodes[msg->sender_idx].name);
         }
     }
@@ -843,7 +882,8 @@ gcs_group_handle_last_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
     log_debug << "Got last applied " << gtid.seqno() << " from "
               << msg->sender_idx << " (" << group->nodes[msg->sender_idx].name
               << "). Last node: " << group->last_node << " ("
-              << group->nodes[group->last_node].name << ")";
+              << (group->last_node >= 0 ?
+                  group->nodes[group->last_node].name : " ") << ")";
 
     if (msg->sender_idx == group->last_node   &&
         gtid.seqno()    >  group->last_applied) {
@@ -854,7 +894,7 @@ gcs_group_handle_last_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
         group_redo_last_applied (group);
 
         if (old_val < group->last_applied) {
-            gu_debug ("New COMMIT CUT %lld on %d after %lld from %d",
+            gu_debug ("New COMMIT CUT %lld on %ld after %lld from %d",
                       (long long)group->last_applied, group->my_idx,
                       (long long)gtid.seqno(), msg->sender_idx);
             return group->last_applied;
@@ -868,7 +908,7 @@ gcs_group_handle_last_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
 static inline bool
 group_count_votes(const gcs_node_t& node)
 {
-    return (node.count_last_applied && !node.arbitrator);
+    return (node.count_last_applied && !node.stateless);
 }
 
 /* true if last vote was updated, false if not */
@@ -1003,7 +1043,7 @@ group_recount_votes (gcs_group_t& group)
         // record voting result in the history for later
         std::pair<gu::GTID,int64_t> const val(vote_gtid, win_vote);
         std::pair<VoteHistory::iterator, bool> const res
-                    (group.vote_history->insert(val));
+                    (group.vote_history.insert(val));
         if (false == res.second)
         {
             assert(0);
@@ -1053,8 +1093,12 @@ gcs_group_handle_vote_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
                  << gtid << ',' << gu::PrintBase<>(code) << ": "
                  << (code ? (data ? data : "(null)") : "Success");
 
-        gcs_node_set_vote (&sender, gtid.seqno(), code,
-                           group->quorum.gcs_proto_ver);
+        {
+            gu::Lock lock(group->memb_mtx_);
+            group->memb_epoch_ = group->act_id_;
+            gcs_node_set_vote (&sender, gtid.seqno(), code,
+                               group->quorum.gcs_proto_ver);
+        }
 
         if (group_recount_votes(*group))
         {
@@ -1086,11 +1130,11 @@ gcs_group_handle_vote_msg (gcs_group_t* group, const gcs_recv_msg_t* msg)
         msg << "Recovering vote result from history: " << gtid;
 
         int64_t result(0);
-        VoteHistory::iterator it(group->vote_history->find(gtid));
-        if (group->vote_history->end() != it)
+        VoteHistory::iterator it(group->vote_history.find(gtid));
+        if (group->vote_history.end() != it)
         {
             result = it->second;
-            group->vote_history->erase(it);
+            group->vote_history.erase(it);
             msg << ',' << gu::PrintBase<>(result);
         }
         else
@@ -1200,15 +1244,16 @@ gcs_group_handle_join_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
             }
         }
 
-        if (j == group->num) {
-            gu_warn ("Could not find peer: %s", peer_id);
+        if (j == group->num && strlen(peer_id)) {
+            /* This can happen if the 'peer' is no longer in group. */
+            gu_info ("Could not find peer: %s", peer_id);
         }
 
         if (code < 0) {
-            gu_warn ("%d.%d (%s): State transfer %s %d.%d (%s) failed: %d (%s)",
+            gu_warn ("%d.%d (%s): State transfer %s %d.%d (%s) failed: %s",
                      sender_idx, sender->segment, sender->name, st_dir,
                      peer_idx, peer ? peer->segment : -1, peer_name,
-                     (int)code, strerror((int)-code));
+                     gcs_state_transfer_error_str((int)-code));
 
             if (from_donor && peer_idx == group->my_idx &&
                 GCS_NODE_STATE_JOINER == group->nodes[peer_idx].status) {
@@ -1250,8 +1295,14 @@ gcs_group_handle_join_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
             gu_warn("Rejecting JOIN message from %d.%d (%s): new State Transfer"
                     " required.", sender_idx, sender->segment, sender->name);
         }
-        else {
-            // should we freak out and throw an error?
+        else if (GCS_NODE_STATE_SYNCED != sender->status &&
+                 GCS_NODE_STATE_JOINED != sender->status) {
+            /* According to comments in gcs_join(), sending of JOIN messages
+             * is always allowed when not in JOINER state. This may lead to
+             * duplicate joins of which some can be received in JOINED or
+             * SYNCED state. This is expected, so the warning is not printed if
+             * the state is JOINED or SYNCED, but we'll keep it for other
+             * states to catch possible errors in sender logic. */
             gu_warn("Protocol violation. JOIN message sender %d.%d (%s) is not "
                     "in state transfer (%s). Message ignored.",
                     sender_idx, sender->segment, sender->name,
@@ -1286,7 +1337,7 @@ gcs_group_handle_sync_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
         group->memb_epoch_ = group->act_id_;
 
         sender->status = GCS_NODE_STATE_SYNCED;
-        sender->count_last_applied = group_count_arbitrator(*group, *sender);
+        sender->count_last_applied = group_count_stateless(*group, *sender);
 
         group_redo_last_applied (group); //from now on this node must be counted
 
@@ -1323,7 +1374,7 @@ group_node_is_stateful (const gcs_group_t* group, const gcs_node_t* node)
         return strcmp (node->name, GCS_ARBITRATOR_NAME);
     }
     else {
-        return ((gcs_node_flags(node) & GCS_STATE_ARBITRATOR) == 0);
+        return (!node->stateless);
     }
 }
 
@@ -1343,7 +1394,9 @@ group_find_node_by_state (const gcs_group_t* const group,
 
         gcs_node_t* node = &group->nodes[idx];
 
-        if (node->status >= status && group_node_is_stateful (group, node))
+        if (!group_node_is_stateful(group, node)) continue;
+
+        if (node->status >= status)
         {
             donor = idx; /* potential donor */
         }
@@ -1358,7 +1411,7 @@ group_find_node_by_state (const gcs_group_t* const group,
     /* Have not found suitable donor in the same segment. */
     if (!hnss && donor >= 0) {
         if (joiner_idx == group->my_idx) {
-            gu_warn ("There are no nodes in the same segment that will ever "
+            gu_info ("There are no nodes in the same segment that will ever "
                      "be able to become donors, yet there is a suitable donor "
                      "outside. Will use that one.");
         }
@@ -1379,6 +1432,9 @@ group_find_node_by_name (const gcs_group_t* const group, int const joiner_idx,
 
     for (idx = 0; idx < group->num; idx++) {
         gcs_node_t* node = &group->nodes[idx];
+
+        if (!group_node_is_stateful(group, node)) continue;
+
         if (!strncmp(node->name, name, name_len)) {
             if (joiner_idx == idx) {
                 return -EHOSTDOWN;
@@ -1700,7 +1756,6 @@ gcs_group_find_donor(const gcs_group_t* group,
     return donor_idx;
 }
 
-
 /*!
  * Selects and returns the index of state transfer donor, if available.
  * Updates donor and joiner status if state transfer is possible
@@ -1780,12 +1835,24 @@ group_select_donor (gcs_group_t* group,
             assert(true == desync);
         }
     }
-    else {
-        gu_warn ("Member %d.%d (%s) requested state transfer from '%s', "
-                 "but it is impossible to select State Transfer donor: %s",
-                 joiner_idx, group->nodes[joiner_idx].segment,
-                 group->nodes[joiner_idx].name,
-                 required_donor ? donor_string : "*any*", strerror (-donor_idx));
+    else if (-donor_idx == EAGAIN) {
+        /* In case of EAGAIN the failure of selecting the donor is
+         * transient, and donor selection may succeed when the request is
+         * retried by the Joiner. Therefore print info level message
+         * instead of warning. */
+        gu_info("Member %d.%d (%s) requested state transfer from '%s', "
+                "but it is impossible to select State Transfer donor: %s",
+                joiner_idx, group->nodes[joiner_idx].segment,
+                group->nodes[joiner_idx].name,
+                required_donor ? donor_string : "*any*",
+                gcs_state_transfer_error_str(-donor_idx));
+    } else {
+        gu_warn("Member %d.%d (%s) requested state transfer from '%s', "
+                "but it is impossible to select State Transfer donor: %s",
+                joiner_idx, group->nodes[joiner_idx].segment,
+                group->nodes[joiner_idx].name,
+                required_donor ? donor_string : "*any*",
+                gcs_state_transfer_error_str(-donor_idx));
     }
 
     return donor_idx;
@@ -1796,7 +1863,7 @@ void
 gcs_group_ignore_action (gcs_group_t* group, struct gcs_act_rcvd* act)
 {
     gu_debug("Ignoring action: buf: %p, len: %zd, type: %d, sender: %d, "
-             "seqno: %lld", act->act.buf, act->act.buf_len, act->act.type,
+             "seqno: %" PRId64, act->act.buf, act->act.buf_len, act->act.type,
              act->sender_idx, act->id);
 
     if (gcs_act_in_cache(act->act.type)) {
@@ -2048,20 +2115,26 @@ gcs_group_act_conf (gcs_group_t*         group,
     return rcvd->act.buf_len;
 }
 
-// for future use in fake state exchange (in unit tests et.al. See #237, #238)
-static gcs_state_msg_t*
-group_get_node_state (const gcs_group_t* const group, long const node_idx)
+static uint8_t
+group_node_flags(const gcs_node_t* const node, int const node_idx)
 {
-    const gcs_node_t* const node = &group->nodes[node_idx];
-
-    uint8_t flags = 0;
-
+    uint8_t flags(0);
     if (0 == node_idx)            flags |= GCS_STATE_FREP;
     if (node->count_last_applied) flags |= GCS_STATE_FCLA;
     if (node->bootstrap)          flags |= GCS_STATE_FBOOTSTRAP;
-#ifdef GCS_FOR_GARB
-    flags |= GCS_STATE_ARBITRATOR;
+    if (node->stateless)          flags |= GCS_STATE_FSTATELESS;
+    return flags;
+}
 
+// for future use in fake state exchange (in unit tests et.al. See #237, #238)
+static gcs_state_msg_t*
+group_get_node_state (const gcs_group_t* const group, int const node_idx)
+{
+    const gcs_node_t* const node = &group->nodes[node_idx];
+
+    uint8_t flags(group_node_flags(node, node_idx));
+
+#ifdef GCS_FOR_GARB
     int64_t const cached = GCS_SEQNO_ILL;
 #else
     int64_t const cached = /* group->cache check is needed for unit tests */
@@ -2114,6 +2187,12 @@ gcs_group_param_set(gcs_group_t& group,
             "cluster.";
     }
 
+    if (GCS_STATELESS_KEY == key)
+    {
+        gu_throw_error(ENOTSUP) << "Setting '" << key << "' in runtime may "
+            "have unintended consequences and is currently not supported.";
+    }
+
     return 1;
 }
 
@@ -2137,10 +2216,34 @@ int gcs_group_get_status (const gcs_group_t* group, gu::Status& status)
     return 0;
 }
 
+static void
+group_membership_set_flags(struct wsrep_member_info_ext&,
+                           uint8_t)
+{}
+
+static void
+group_membership_set_flags(struct wsrep_member_info_ext_v2& mn,
+                           uint8_t const gcs_flags)
+{
+    uint8_t wsrep_flags(0);
+
+    if (gcs_flags & GCS_STATE_FREP)
+        wsrep_flags |= WSREP_MEMBER_FLAGS_REP;
+    if (gcs_flags & GCS_STATE_FCLA)
+        wsrep_flags |= WSREP_MEMBER_FLAGS_CLA;
+    if (gcs_flags & GCS_STATE_FBOOTSTRAP)
+        wsrep_flags |= WSREP_MEMBER_FLAGS_BOOTSTRAP;
+    if (gcs_flags & GCS_STATE_FSTATELESS)
+        wsrep_flags |= WSREP_MEMBER_FLAGS_STATELESS;
+
+    mn.flags = wsrep_flags;
+}
+
+template <typename M>
 void
-gcs_group_get_membership(const gcs_group_t&        group,
-                         wsrep_allocator_cb const  alloc,
-                         struct wsrep_membership** memb)
+group_get_membership(const gcs_group_t&       group,
+                     wsrep_allocator_cb const alloc,
+                     M** memb)
 {
     if (!alloc)
     {
@@ -2150,10 +2253,9 @@ gcs_group_get_membership(const gcs_group_t&        group,
     gu::Lock lock(group.memb_mtx_);
 
     size_t const memb_size
-        (sizeof(struct wsrep_membership) +
-         (group.num - 1)*sizeof(struct wsrep_member_info_ext));
+        (sizeof(M) + (group.num - 1)*sizeof(decltype((*memb)->members[0])));
 
-    *memb = static_cast<struct wsrep_membership*>(alloc(memb_size));
+    *memb = static_cast<M*>(alloc(memb_size));
 
     if (!*memb)
     {
@@ -2163,7 +2265,7 @@ gcs_group_get_membership(const gcs_group_t&        group,
 
     ::memset(*memb, 0, memb_size);
 
-    struct wsrep_membership& m(**memb);
+    M& m(**memb);
 
     m.group_uuid = group.group_uuid;
     m.updated = group.memb_epoch_;
@@ -2187,7 +2289,7 @@ gcs_group_get_membership(const gcs_group_t&        group,
 
     for (size_t i(0); i < m.num; ++i)
     {
-        struct wsrep_member_info_ext& mn(m.members[i]);
+        auto& mn(m.members[i]);
         const struct gcs_node& gn(group.nodes[i]);
 
         gu_uuid_t uuid;
@@ -2222,8 +2324,27 @@ gcs_group_get_membership(const gcs_group_t&        group,
         case GCS_NODE_STATE_MAX:
             mn.status = WSREP_MEMBER_ERROR;
         }
+
+        uint8_t const gcs_flags(group_node_flags(&group.nodes[i], i));
+        group_membership_set_flags(mn, gcs_flags);
     }
 }
+
+void
+gcs_group_get_membership(const gcs_group_t&        group,
+                         wsrep_allocator_cb const  alloc,
+                         struct wsrep_membership** memb)
+{
+    group_get_membership(group, alloc, memb);
+}
+
+void
+gcs_group_get_membership(const gcs_group_t&           group,
+                         wsrep_allocator_cb const     alloc,
+                         struct wsrep_membership_v2** memb)
+{
+    group_get_membership(group, alloc, memb);
+}
 int gcs_group_fetch_pfs_info(const gcs_group_t*  group,
                              wsrep_node_info_t** nodes_arg,
                              uint32_t*           size,
