@@ -6,10 +6,16 @@
 #ifndef _gcs_params_h_
 #define _gcs_params_h_
 
+#include "gu_config.hpp"
 #include "galerautils.h"
 
 struct gcs_params
 {
+    gcs_params(gu::Config& config);
+
+    static void
+    register_params(gu::Config& config);
+
     double  fc_resume_factor;
     double  recv_q_soft_limit;
     double  max_throttle;
@@ -34,16 +40,5 @@ extern const char* const GCS_PARAMS_MAX_THROTTLE;
 extern const char* const GCS_PARAMS_SM_DUMP;
 #endif /* GCS_SM_DEBUG */
 
-/*! Register configuration parameters */
-extern bool
-gcs_params_register(gu_config_t* config);
-
-/*! Initializes parameters from config
- *
- * @return 0 in case of success,
- *         -EINVAL if some values were set incorrectly in config */
-extern long
-gcs_params_init (struct gcs_params* params, gu_config_t* config);
-
 #endif /* _gcs_params_h_ */
 
